from safe_url_checker.integrations.base_integration import CheckResult
from safe_url_checker.integrations import GoogleSafeBase, GoogleSafeInterstitial, GoogleSafeTransparency, Virustotal


INTEGRATIONS = {
    'GOOGLE_SAFE_BASE': GoogleSafeBase,
    'GOOGLE_SAFE_INTERSTITIAL': GoogleSafeInterstitial,
    'GOOGLE_SAFE_TRANSPARENCY': GoogleSafeTransparency,
    'VIRUSTOTAL': Virustotal,
}

class IntegrationNotFound(Exception):
    def __init__(self, integration: str):
        super().__init__(f"Integration not found: {integration}")

class IntegrationMissingParam(Exception):
    def __init__(self, integration: str, param: str):
        super().__init__(f"Integration {integration} requires param: {param}")

class SafeUrlChecker:
    def __init__(self) -> None:
        self.selected_integrations = []
        self.params = {}

    def _get_integration(self, integration: str):
        if integration in INTEGRATIONS:
            return INTEGRATIONS[integration]
        raise IntegrationNotFound(integration)

    def get_integrations_list(self) -> list[str]:
        return list(INTEGRATIONS.keys())

    def set_integrations(self, integrations: list[str]) -> None:
        for integration in integrations:
            if integration not in INTEGRATIONS:
                raise IntegrationNotFound(integration)
        self.selected_integrations = integrations

    def set_integration_params(self, integration: str, params: dict[str, str]) -> None:
        if integration not in INTEGRATIONS:
            raise IntegrationNotFound(integration)

        self.params[integration] = params

    def check(self, urls: list[str]) -> dict[str, CheckResult]:
        result = {}

        for integration in self.selected_integrations:

            integration_class = self._get_integration(integration)
            integration_class = integration_class(self.params.get(integration, {}))

            required_params = integration_class.required_params()
            for param in required_params:
                if integration not in self.params or param not in self.params[integration]:
                    raise IntegrationMissingParam(integration, param)

            result[integration] = integration_class.test(list(dict.fromkeys(urls)))

        return result



from safe_url_checker.integrations.base_integration import BaseIntegration, CheckResult, UrlCheckResult

import time
import requests


class GoogleSafeBase(BaseIntegration):
    def required_params(self) -> list[str]:
        return ['API_KEY']

    def test(self, urls: list[str]) -> CheckResult:
        result: CheckResult = []

        if not urls:
            return result

        chunk_size = self.params.get('CHUNK_SIZE', 500)
        while urls:
            chunk, urls = urls[:chunk_size], urls[chunk_size:]
            chunk = [{'url': u} for u in chunk]

            response = requests.post(
                'https://safebrowsing.googleapis.com/v4/threatMatches:find',
                params={
                    'key': self.params['API_KEY'],
                },
                json={
                    'threatInfo': {
                        'threatTypes': self.params.get('THREAT_TYPES', ['SOCIAL_ENGINEERING', 'MALWARE', 'UNWANTED_SOFTWARE', 'POTENTIALLY_HARMFUL_APPLICATION']),
                        'platformTypes': self.params.get('PLATFORM_TYPES', ['ANY_PLATFORM']),
                        'threatEntryTypes': ['URL'],
                        'threatEntries': chunk
                    }
                }
            )

            if response.status_code != 200:
                continue

            response = response.json()

            not_safe = [r['threat']['url'] for r in response['matches']] if 'matches' in response else []

            for url in chunk:
                result.append(UrlCheckResult(safe=url['url'] not in not_safe, url=url['url']))

            time.sleep(self.params.get('SLEEP_TIME', 1))

        return result



from safe_url_checker.integrations.base_integration import BaseIntegration, CheckResult, UrlCheckResult
from ssl import SSLError

import requests
import random


class GoogleSafeTransparency(BaseIntegration):
    def required_params(self) -> list[str]:
        return []

    def test(self, urls: list[str]) -> CheckResult:
        result: CheckResult = []

        if not urls:
            return result

        for url in urls:
            try:
                response = requests.get(
                    f"https://transparencyreport.google.com/transparencyreport/api/v3/safebrowsing/status?site={url}",
                    headers={
                        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (HTML, like Gecko) Chrome/102.0.0.0 Safari/537.36'
                    },
                    timeout=random.randint(10, 15)
                )

                if response.status_code == 200:
                    if 'sb.ssr' in response.text:
                        if 'true' not in response.text:
                            result.append(UrlCheckResult(safe=True, url=url))
                            continue

                    result.append(UrlCheckResult(safe=False, url=url))

            except SSLError as e:
                continue

        return result



from .google_safe_base import GoogleSafeBase
from .google_safe_interstitial import GoogleSafeInterstitial
from .google_safe_transparency import GoogleSafeTransparency
from .virustotal import Virustotal


__all__ = ["GoogleSafeBase", "GoogleSafeInterstitial", "GoogleSafeTransparency", "Virustotal"]



from abc import ABC, abstractmethod
from typing import TypedDict


class UrlCheckResult(TypedDict):
    safe: bool
    url: str

CheckResult = list[UrlCheckResult]

class BaseIntegration(ABC):

    def __init__(self, params):
        self.params = params or {}

    @abstractmethod
    def required_params(self) -> list[str]:
        pass

    @abstractmethod
    def test(self, urls: list[str]) -> CheckResult:
        pass



from safe_url_checker.integrations.base_integration import BaseIntegration, CheckResult, UrlCheckResult
from datetime import datetime

import requests
import time


class Virustotal(BaseIntegration):
    def required_params(self) -> list[str]:
        return ['API_KEY']

    def test(self, urls: list[str]) -> CheckResult:
        result: CheckResult = []

        if not urls:
            return result

        i = 0
        count_requests = 0

        api_url = 'https://www.virustotal.com/api/v3/urls'
        headers = {'x-apikey': self.params['API_KEY']}

        for url in urls:
            data = {'url': url}

            while True:
                if count_requests >= 4:
                    i += 1
                    count_requests = 0

                response = requests.post(api_url, headers=headers, data=data)
                count_requests += 1

                if response.status_code in [429, 401]:
                    time.sleep(60)
                    continue
                else:
                    break

            resp = response.json()

            virustotal_id = resp['data']['id'].split('-')[1]

            while True:
                if count_requests >= 4:
                    i += 1
                    count_requests = 0

                response = requests.get(api_url + '/' + virustotal_id, headers=headers)
                count_requests += 1

                if response.status_code in [429, 401]:
                    time.sleep(60)
                    continue
                else:
                    break

            data = response.json()

            if 'error' in data:
                continue

            report = {}

            report['url'] = url

            virustotal = True

            if 'data' in data and 'attributes' in data['data']:
                data = data['data']['attributes']
                report['stats'] = data['total_votes']
                report['results'] = data['last_analysis_results']

                if 'last_analysis_date' in data:
                    report['last_update'] = datetime.utcfromtimestamp(int(data['last_analysis_date'])).strftime('%Y-%m-%d %H:%M:%S')

                for treat in report['results']:
                    treat_result = report['results'][treat]['result']
                    if treat_result not in ["clean", "unrated"]:
                        virustotal = False

            result.append(UrlCheckResult(safe=virustotal, url=url))

        return result



from safe_url_checker.integrations.base_integration import BaseIntegration, CheckResult, UrlCheckResult
from ssl import SSLError

import requests


class GoogleSafeInterstitial(BaseIntegration):
    def required_params(self) -> list[str]:
        return []

    def test(self, urls: list[str]) -> CheckResult:
        result: CheckResult = []

        if not urls:
            return result

        for url in urls:
            try:
                response = requests.get(f"https://www.google.com/interstitial?url={url}")

                if response.status_code == 200:
                    result.append(UrlCheckResult(safe=False, url=url))
                else:
                    result.append(UrlCheckResult(safe=True, url=url))

            except SSLError as e:
                continue

        return result



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['safe_url_checker', 'safe_url_checker.integrations']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'safe-url-checker',
    'version': '1.0.1',
    'description': 'A library for checking links for security using several different integrations.',
    'long_description': '# Installation\n```\npoetry add safe_url_checker\n```\n\n# Description\nA library for checking links for security using several different integrations.\n\n# How to use:\n```\nfrom safe_url_checker import SafeUrlChecker\n\nchecker = SafeUrlChecker()\nchecker.set_integrations([\n    \'GOOGLE_SAFE_BASE\',\n    \'GOOGLE_SAFE_INTERSTITIAL\',\n    \'GOOGLE_SAFE_TRANSPARENCY\',\n    \'VIRUSTOTAL\',\n])\nchecker.set_integration_params(\'GOOGLE_SAFE_BASE\', {\n    \'API_KEY\': \'token\'\n})\nchecker.set_integration_params("VIRUSTOTAL", {\n    \'API_KEY\': \'token\'\n})\ncheck_result = checker.check(urls=[\n    \'https://google.com\',\n])\n```\n',
    'author': 'Aleksei Kovalenko',
    'author_email': 'kalexhaym@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
